import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;



public class ProxyToServer {

	private final static int HEADER_SIZE = 2;
	private final static int DATA_HEADER_SIZE = 4;	// LENGTH_DATA(4)

	//public final static String SUCCESS = "1";
	public final static char SUCCESS = '1';

	public final static int SEND_DATA_SIZE = 1024;
	public final static int RECV_DATA_SIZE = 128;


	public ProxyToServer() {
		
	}
	
	public Socket initSocket(String ip, int port) throws Exception {
		return new Socket(ip, port);
	}
	
	public void clearSocket(Socket sock) {
		try {
			sock.close();
			sock = null;
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public void sendRequestPcmPlayer(Socket sock, int cmd, int subCmd, InputStream stream) throws IOException {
		if(stream != null) {
			
			FileInputStream fileIS = (FileInputStream) stream;
			BufferedInputStream bufferedIS = new BufferedInputStream(fileIS);
			OutputStream sockOutputStream = sock.getOutputStream();
			BufferedOutputStream bufferedOS = new BufferedOutputStream(sockOutputStream);

			int lenRead, lenTotalRead;
			byte[] bufData = new byte[SEND_DATA_SIZE];
			
			try {
				// Step 1. send Header
				bufferedOS.write(String.format("%d%d", cmd, subCmd).getBytes());
				bufferedOS.flush();
				
				// Step 2. send DataLength + Data
				lenTotalRead = 0;
				do {
					if ((lenRead = bufferedIS.read(bufData, 0, SEND_DATA_SIZE)) > 0) {
						bufferedOS.write(String.format("%04d", lenRead).getBytes(), 0, DATA_HEADER_SIZE);
						bufferedOS.write(bufData, 0, lenRead);
						bufferedOS.flush();

						lenTotalRead += lenRead;
						System.out.println("bufData(" + lenRead + "): " + bufData);
					}
					else {
						System.out.println("End of read");
						break;
					}
				} while (true);

				bufferedOS.write(String.format("%04d", 0).getBytes(), 0, DATA_HEADER_SIZE);
				bufferedOS.flush();
				
				System.out.println("lenTotalRead: " + lenTotalRead);
				System.out.println("send completed");
			}
			finally {
				bufferedIS.close();
				bufferedIS = null;
				bufferedOS.close();
				bufferedOS = null;
				sockOutputStream.close();
				sockOutputStream = null;
			}
		}
	}
	
	public byte[] recvResponse(Socket sock, int cmd, int subCmd) throws IOException {
		byte[] bufHeader = new byte[DATA_HEADER_SIZE+1];
		byte[] bufData = new byte[RECV_DATA_SIZE * 5 + 1];
		int totalDataSize, nowDataSize, nowRecv, nowTotalRecv;
		
		InputStream sockInputStream = sock.getInputStream();
		
		try {
			// Step 1. recv Header
			nowRecv = sockInputStream.read(bufData, 0, HEADER_SIZE);
			if(nowRecv < HEADER_SIZE) {
				System.out.println("[ERROR] recvResponse - Fail to recv header.\n");
				return new byte[0];
			}
		
			if((Integer.parseInt(new String(bufData, 0, 1)) != cmd)
					|| (Integer.parseInt(new String(bufData, 1, 1)) != subCmd)) {
				System.out.println("[ERROR] recvResponse - Invalid response for request.\n");
				return new byte[0];
			}
		
			// Step 2. recv DataLength + Data
			totalDataSize = 0;
		
			do {
				nowRecv = sockInputStream.read(bufHeader, 0, DATA_HEADER_SIZE);
				if(nowRecv < DATA_HEADER_SIZE) {
					System.out.println("[ERROR] recvResponse - Fail to recv head of data part.\n");
					return new byte[0];
				}
			
				nowDataSize = Integer.parseInt(new String(bufHeader, 0, 4));
				if(nowDataSize == 0)
					break;

				nowTotalRecv = 0;
				while(nowTotalRecv < totalDataSize) {
					nowRecv = sockInputStream.read(bufData, totalDataSize+nowTotalRecv, nowDataSize-nowTotalRecv);
				
					nowTotalRecv += nowRecv;
				}
			
				totalDataSize += nowTotalRecv;
			} while(true);
		}
		finally {
			sockInputStream.close();
			sockInputStream = null;
		}
		
		System.out.println("recv completed");
		
		return bufData;
	}
}
